@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-dialog.c,v 1.10 2003/01/30 18:34:12 cwalters Exp $
+ *  $Id: rb-dialog.c,v 1.11 2003/08/17 08:14:45 cwalters Exp $
  */
 
 #include <config.h>
@@ -174,7 +174,7 @@ rb_ask_string (const char *question,
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-	image = gtk_image_new_from_stock (RB_STOCK_GROUP,
+	image = gtk_image_new_from_stock (RB_STOCK_PLAYLIST,
 					  GTK_ICON_SIZE_DIALOG);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
 
