@@ -323,6 +323,9 @@ rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
+/* VOID:FLOAT (./rb-marshal.list:7) */
+#define rb_marshal_VOID__FLOAT	g_cclosure_marshal_VOID__FLOAT
+
 G_END_DECLS
 
 #endif /* __rb_marshal_MARSHAL_H__ */
