@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-library.c,v 1.67 2003/08/15 05:20:11 cwalters Exp $
+ *  $Id: rb-library.c,v 1.69 2003/08/19 02:56:42 cwalters Exp $
  */
 
 #include <config.h>
@@ -59,14 +59,21 @@ static void restore_node (RBNode *node);
 static gboolean rb_library_periodic_save (RBLibrary *library);
 static void sync_sort_keys (RBNode *node);
 static void walker_thread_done_cb (RBLibraryWalkerThread *thread, RBLibrary *library);
-static char *get_status_fast (RBLibrary *library, RBNode *root);
-static char *get_status_full (RBLibrary *library, RBNode *root, RBNodeFilter *filter);
+static char *get_status_normal (RBLibrary *library, RBNode *root, RBNodeFilter *filter);
+static gboolean poll_status_update (gpointer data);
 
+enum RBLibraryState
+{
+	LIBRARY_STATE_NONE,
+	LIBRARY_STATE_INITIAL_REFRESH,
+};
 
 struct RBLibraryPrivate
 {
 	GMutex *lock;
 
+	enum RBLibraryState state;
+
 	RBLibraryWalkerThread *walker_thread;
 	RBLibraryMainThread *main_thread;
 
@@ -74,6 +81,7 @@ struct RBLibraryPrivate
 	GList *walker_threads;
 
 	GAsyncQueue *main_queue;
+	GAsyncQueue *add_queue;
 
 	RBNodeDb *db;
 
@@ -99,6 +107,8 @@ struct RBLibraryPrivate
 
 	char *xml_file;
 
+	guint refresh_count;
+
 	guint idle_save_id;
 
 	gboolean status_poll_queued;
@@ -116,6 +126,7 @@ enum
 {
 	ERROR,
 	STATUS_CHANGED,
+	PROGRESS,
 	LAST_SIGNAL,
 };
 
@@ -180,6 +191,15 @@ rb_library_class_init (RBLibraryClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	rb_library_signals[PROGRESS] =
+		g_signal_new ("progress",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBLibraryClass, progress),
+			      NULL, NULL,
+			      rb_marshal_VOID__FLOAT,
+			      G_TYPE_NONE,
+			      1, G_TYPE_FLOAT);
 }
 
 static void
@@ -220,9 +240,12 @@ rb_library_init (RBLibrary *library)
 
 	rb_library_create_skels (library);
 
+	library->priv->lock = g_mutex_new ();
+
 	library->priv->walker_mutex = g_mutex_new ();
 
 	library->priv->main_queue = g_async_queue_new ();
+	library->priv->add_queue = g_async_queue_new ();
 }
 
 static void
@@ -251,6 +274,17 @@ rb_library_release_brakes (RBLibrary *library)
 						  library);
 }
 
+static gboolean
+queue_is_empty (GAsyncQueue *queue)
+{
+	gpointer data = g_async_queue_try_pop (queue);
+	if (data == NULL)
+		return TRUE;
+
+	g_async_queue_push (queue, data);
+	return FALSE;
+}
+
 /* We don't particularly care about race conditions here.  This function
  * is just supposed to give some feedback about whether the library
  * is busy or not, it doesn't have to be precise.
@@ -258,12 +292,8 @@ rb_library_release_brakes (RBLibrary *library)
 gboolean
 rb_library_is_idle (RBLibrary *library)
 {
-	gpointer data = g_async_queue_try_pop (library->priv->main_queue);
-	if (data == NULL)
-		return TRUE;
-
-	g_async_queue_push (library->priv->main_queue, data);
-	return FALSE;
+	return queue_is_empty (library->priv->main_queue)
+		&& queue_is_empty (library->priv->add_queue);
 }
 
 void
@@ -305,6 +335,7 @@ rb_library_finalize (GObject *object)
 	g_mutex_free (library->priv->walker_mutex);
 
 	g_async_queue_unref (library->priv->main_queue);
+	g_async_queue_unref (library->priv->add_queue);
 
 	rb_library_save (library);
 
@@ -326,6 +357,8 @@ rb_library_finalize (GObject *object)
 	g_static_rw_lock_free (library->priv->album_hash_lock);
 	g_static_rw_lock_free (library->priv->song_hash_lock);
 
+	g_mutex_free (library->priv->lock);
+
 	g_free (library->priv->cached_status);
 
 	g_free (library->priv->xml_file);
@@ -354,9 +387,45 @@ rb_library_new (void)
 static void
 signal_status_changed (RBLibrary *library)
 {
-	g_get_current_time (&library->priv->mod_time);
+	rb_thread_helpers_lock_gdk ();
+
+	if (!library->priv->in_shutdown) {
+		g_get_current_time (&library->priv->mod_time);
+		
+		g_signal_emit (G_OBJECT (library), rb_library_signals[STATUS_CHANGED], 0);
+	}
 	
-	g_signal_emit (G_OBJECT (library), rb_library_signals[STATUS_CHANGED], 0);
+	rb_thread_helpers_unlock_gdk ();
+}
+
+static void
+signal_progress_changed (RBLibrary *library)
+{
+	rb_thread_helpers_lock_gdk ();
+
+	if (library->priv->in_shutdown)
+		goto out_unlock;
+
+	if (!queue_is_empty (library->priv->add_queue)) {
+		g_signal_emit (G_OBJECT (library), rb_library_signals[PROGRESS], 0, -1.0);
+	} else if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
+		float total;
+		float refresh_count;
+
+		g_mutex_lock (library->priv->lock);
+		refresh_count = (float) library->priv->refresh_count;
+		g_mutex_unlock (library->priv->lock);
+
+		total = (float) rb_node_get_n_children (library->priv->all_songs);
+		
+		g_signal_emit (G_OBJECT (library), rb_library_signals[PROGRESS], 0,
+			       refresh_count / total);
+	} else {
+		g_signal_emit (G_OBJECT (library), rb_library_signals[PROGRESS], 0, 1.0);
+	}
+
+out_unlock:
+	rb_thread_helpers_unlock_gdk ();
 }
 
 static void
@@ -371,13 +440,16 @@ walker_thread_done_cb (RBLibraryWalkerThread *thread, RBLibrary *library)
 	g_mutex_unlock (library->priv->walker_mutex);
 }
 
+/* MULTI-THREAD ENTRYPOINT
+ * Locks required: None
+ */
 void
 rb_library_add_uri (RBLibrary *library, const char *uri)
 {
 	if (rb_uri_is_directory (uri) == FALSE) {
 		RBLibraryAction *action = rb_library_action_new (RB_LIBRARY_ACTION_ADD_FILE, uri);
 		rb_debug ("queueing ADD_FILE for %s", uri);
-		g_async_queue_push (library->priv->main_queue, action);
+		g_async_queue_push (library->priv->add_queue, action);
 	} else {
 		RBLibraryWalkerThread *thread = rb_library_walker_thread_new (library, uri);
 
@@ -393,6 +465,9 @@ rb_library_add_uri (RBLibrary *library, const char *uri)
 	}
 }
 
+/* MULTI-THREAD ENTRYPOINT
+ * Locks required: None
+ */
 void
 rb_library_add_uri_sync (RBLibrary *library, const char *uri, GError **error)
 {
@@ -400,43 +475,56 @@ rb_library_add_uri_sync (RBLibrary *library, const char *uri, GError **error)
 		rb_debug ("uri \"%s\" does not exist, adding new node", uri);
 		rb_library_new_node (library, uri, error);
 		if (error && *error)
-			return;
+			goto signal;
 	} else {
 		rb_debug ("uri \"%s\" already exists", uri);
 	}
 	
 	rb_file_monitor_add (rb_file_monitor_get (), uri);
 
-	rb_thread_helpers_lock_gdk ();
-	if (!library->priv->in_shutdown)
+signal:
+	if (!library->priv->in_shutdown) {
 		signal_status_changed (library);
-	rb_thread_helpers_unlock_gdk ();
+		signal_progress_changed (library);
+	}
 }
 
+
+/* MULTI-THREAD ENTRYPOINT
+ * Locks required: None
+ */
 gboolean
 rb_library_update_uri (RBLibrary *library, const char *uri, GError **error)
 {
 	RBNode *song;
-	gboolean ret;
+	gboolean ret = FALSE;
+
+	g_mutex_lock (library->priv->lock);
+	library->priv->refresh_count++;
+	g_mutex_unlock (library->priv->lock);
 	
 	song = rb_library_get_song_by_location (library, uri);
 	if (song == NULL)
-		return FALSE;
+		goto out;
 	
 	if (rb_uri_exists (uri) == FALSE) {
 		rb_debug ("song \"%s\" was deleted", uri);
 		rb_node_unref (song);
-		return FALSE;
+		goto out;
 	}
 	
 	rb_debug ("updating existing node \"%s\"", uri);
 	ret = rb_library_update_node (library, song, error);
 	if (error && *error) {
-		return ret;
+		goto out;
 	}
 
 	/* just to be sure */
 	rb_file_monitor_add (rb_file_monitor_get (), uri);
+	ret = TRUE;
+out:
+	signal_status_changed (library);
+	signal_progress_changed (library);
 	return ret;
 }
 
@@ -794,6 +882,12 @@ rb_library_get_main_queue (RBLibrary *library)
 	return library->priv->main_queue;
 }
 
+GAsyncQueue *
+rb_library_get_add_queue (RBLibrary *library)
+{
+	return library->priv->add_queue;
+}
+
 RBNode *
 rb_library_get_genre_by_name (RBLibrary *library,
 			      const char *genre)
@@ -917,6 +1011,8 @@ rb_library_load (RBLibrary *library)
 
 	p = rb_profiler_new ("XML loader");
 
+	library->priv->state = LIBRARY_STATE_INITIAL_REFRESH;
+
 	for (child = root->children; child != NULL; child = child->next) {
 		RBNode *node;
 		const char *location;
@@ -1461,10 +1557,35 @@ rb_library_update_node (RBLibrary *library, RBNode *node, GError **error)
 char *
 rb_library_compute_status (RBLibrary *library, RBNode *root, RBNodeFilter *filter)
 {
-	if (!rb_library_is_idle (library))
-		return get_status_fast (library, root);
-	else
-		return get_status_full (library, root, filter);
+	char *ret = NULL;
+	gboolean adding_songs = !queue_is_empty (library->priv->add_queue);
+
+	if (adding_songs)
+		ret = g_strdup_printf (_("<b>Loading songs...</b>"));
+	else if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
+		guint refresh_count;
+		
+		g_mutex_lock (library->priv->lock);
+		refresh_count = library->priv->refresh_count;
+		g_mutex_unlock (library->priv->lock);
+
+		if (!library->priv->status_poll_queued) {
+			g_idle_add ((GSourceFunc) poll_status_update, library);
+			library->priv->status_poll_queued = TRUE;
+		}
+	
+		ret = g_strdup_printf (_("<b>Refreshing songs...</b>"));
+	}
+
+	if (adding_songs || library->priv->state == LIBRARY_STATE_INITIAL_REFRESH) {
+		if (!library->priv->status_poll_queued) {
+			g_idle_add ((GSourceFunc) poll_status_update, library);
+			library->priv->status_poll_queued = TRUE;
+		}
+		return ret;
+	}
+		
+	return get_status_normal (library, root, filter);
 }
 
 static gboolean
@@ -1479,6 +1600,9 @@ poll_status_update (gpointer data)
 	 */
 	if (rb_library_is_idle (library)) {
 
+		if (library->priv->state == LIBRARY_STATE_INITIAL_REFRESH)
+			library->priv->state = LIBRARY_STATE_NONE;
+
 		library->priv->status_poll_queued = FALSE;
 
 		signal_status_changed (library);
@@ -1494,20 +1618,7 @@ poll_status_update (gpointer data)
 }
 
 static char *
-get_status_fast (RBLibrary *library, RBNode *root)
-{
-	char *ret = g_strdup_printf (_("%ld songs"), (long) rb_node_get_n_children (root));
-
-	if (!library->priv->status_poll_queued) {
-		g_idle_add ((GSourceFunc) poll_status_update, library);
-		library->priv->status_poll_queued = TRUE;
-	}
-	
-	return ret;
-}
-
-static char *
-get_status_full (RBLibrary *library, RBNode *root, RBNodeFilter *filter)
+get_status_normal (RBLibrary *library, RBNode *root, RBNodeFilter *filter)
 {
 	char *ret;
 	float days;
