@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: main.c,v 1.25 2003/08/10 18:40:48 cfergeau Exp $
+ *  $Id: main.c,v 1.28 2003/08/24 17:03:39 cwalters Exp $
  */
 
 #include <config.h>
@@ -33,6 +33,10 @@
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
+#ifdef HAVE_GSTREAMER
+#include <gst/gst.h>
+#include <gst/gconf/gconf.h>
+#endif
 
 #include "rb-shell.h"
 #include "rb-debug.h"
@@ -61,7 +65,6 @@ main (int argc, char **argv)
 	GnomeProgram *program;
 	CORBA_Object object;
 	RBShell *rb_shell;
-	char *old_collate = NULL;
 	gboolean sound_events_borked = FALSE;
 
 	const struct poptOption popt_options[] =
@@ -92,38 +95,13 @@ main (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
+#ifdef HAVE_GSTREAMER	
 	/* rb currently does not work with esd + gnome event sounds enabled */
 	if (eel_gconf_get_boolean ("/desktop/gnome/sound/event_sounds")
-			&& eel_gconf_get_boolean ("/desktop/gnome/sound/enable_esd"))
+			&& eel_gconf_get_boolean ("/desktop/gnome/sound/enable_esd")
+	    && strstr (gst_gconf_get_string ("default/audiosink"), "esdsink"))
 		sound_events_borked = TRUE;
-	
-
-	/* workaround for non utf8 LC_COLLATE */
-	old_collate = g_strdup_printf ("LC_COLLATE=%s",
-				       g_getenv ("LC_COLLATE"));
-	if (old_collate == NULL || strstr (old_collate, "UTF-8") == NULL)
-	{
-		char *lang = NULL, *new_collate;
-		const char *env;
-
-		env = g_getenv ("LANG");
-		if (env == NULL)
-		{
-			env = "C";
-		}
-
-		if (strlen (env) >=5)
-			lang = g_strndup (g_getenv ("LANG"), 5);
-		else
-			lang = g_strdup ("en_US");
-
-		new_collate = g_strdup_printf ("LC_COLLATE=%s.UTF-8",
-					       lang);
-		putenv (new_collate);
-
-		g_free (lang);
-		g_free (new_collate);
-	}
+#endif
 	
 	gdk_threads_init ();
 
@@ -169,10 +147,6 @@ main (int argc, char **argv)
 		rb_handle_cmdline (argv, argc, TRUE);
 	}
 
-	/* restore original collate */
-	putenv (old_collate);
-	g_free (old_collate);
-
 	/* cleanup */
 	CORBA_exception_free (&ev);
 
@@ -193,7 +167,7 @@ sound_error_dialog (gpointer unused)
 
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					 _("Rhythmbox does not currently work if GNOME sound events are enabled."));
+					 _("Rhythmbox does not currently work if GNOME sound events are are enabled and esdsink is in use."));
 	gtk_dialog_run (GTK_DIALOG (dialog));
 
 	bonobo_main_quit ();
