@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-shell-player.c,v 1.69 2003/08/17 02:32:06 cwalters Exp $
+ *  $Id: rb-shell-player.c,v 1.76 2003/08/27 02:26:37 cwalters Exp $
  */
 
 #include <gtk/gtk.h>
@@ -187,6 +187,7 @@ struct RBShellPlayerPrivate
 	GtkWidget *buffering_progress;
 	guint buffering_progress_idle_id;
 
+	GtkTooltips *tooltips;
 	GtkWidget *prev_button;
 	PlayButtonState playbutton_state;
 	GtkWidget *play_pause_stop_button;
@@ -369,13 +370,20 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	hbox = gtk_hbox_new (FALSE, 5);
 
+	player->priv->tooltips = gtk_tooltips_new ();
+	gtk_tooltips_enable (player->priv->tooltips);
+
 	/* Previous button */
 	image = gtk_image_new_from_stock (RB_STOCK_PREVIOUS,
 					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+
 	player->priv->prev_button = gtk_button_new ();
 	gtk_container_add (GTK_CONTAINER (player->priv->prev_button), image);
 	g_signal_connect_swapped (G_OBJECT (player->priv->prev_button),
 				  "clicked", G_CALLBACK (rb_shell_player_do_previous), player);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
+			      GTK_WIDGET (player->priv->prev_button), 
+			      _("Previous"), NULL);
 
 	/* Button images */
 	player->priv->play_image = gtk_image_new_from_stock (RB_STOCK_PLAY,
@@ -402,6 +410,9 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_add (GTK_CONTAINER (player->priv->next_button), image);
 	g_signal_connect_swapped (G_OBJECT (player->priv->next_button),
 				  "clicked", G_CALLBACK (rb_shell_player_do_next), player);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
+			      GTK_WIDGET (player->priv->next_button), 
+			      _("Next"), NULL);
 
 	gtk_box_pack_start (GTK_BOX (hbox), player->priv->prev_button, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), player->priv->play_pause_stop_button, FALSE, TRUE, 0);
@@ -418,6 +429,10 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	player->priv->volume_button = GTK_WIDGET (rb_volume_new ());
 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
+			      GTK_WIDGET (player->priv->volume_button), 
+			      _("Volume Control"), NULL);
+
 	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
 	gtk_container_add (GTK_CONTAINER (alignment), player->priv->volume_button);
 	gtk_box_pack_end (GTK_BOX (player), alignment, FALSE, TRUE, 0);
@@ -473,10 +488,6 @@ rb_shell_player_set_property (GObject *object,
 			RBNodeView *songs = rb_source_get_node_view (player->priv->selected_source);
 			GList *extra_views = rb_source_get_extra_views (player->priv->selected_source);
 
-			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
-							      G_CALLBACK (rb_shell_player_playing_node_removed_cb),
-							      player);
-
 			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
 							      G_CALLBACK (rb_shell_player_nodeview_changed_cb),
 							      player);
@@ -502,11 +513,6 @@ rb_shell_player_set_property (GObject *object,
 			RBNodeView *songs = rb_source_get_node_view (player->priv->selected_source);
 			GList *extra_views = rb_source_get_extra_views (player->priv->selected_source);
 
-			g_signal_connect (G_OBJECT (songs),
-					  "playing_node_removed",
-					  G_CALLBACK (rb_shell_player_playing_node_removed_cb),
-					  player);
-
 			g_signal_connect (G_OBJECT (songs),
 					  "changed",
 					  G_CALLBACK (rb_shell_player_nodeview_changed_cb),
@@ -1121,6 +1127,10 @@ rb_shell_player_set_play_button (RBShellPlayer *player,
 		g_error ("Should not get here!");
 		break;
 	}
+	
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
+			      GTK_WIDGET (player->priv->play_pause_stop_button), 
+			      tlabel, NULL);
 
 	gtk_widget_show_all (GTK_WIDGET (player->priv->play_pause_stop_button));
 
@@ -1184,7 +1194,6 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 	PlayButtonState pstate = PLAY_BUTTON_PLAY;
 	RBSource *source = rb_shell_player_get_playing_node (player) == NULL ?
 		player->priv->selected_source : player->priv->source;
-	RBNodeView *songs = rb_source_get_node_view (source);
 	gboolean have_previous, have_next;
 
 	rb_debug ("syncing with source %p", source);
@@ -1205,8 +1214,11 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 
 	rb_bonobo_set_sensitive (player->priv->component, CMD_PATH_CURRENT_SONG,
 				 rb_shell_player_get_playing_node (player) != NULL);
-	rb_bonobo_set_sensitive (player->priv->component, CMD_PATH_SONG_INFO,
-				 rb_node_view_have_selection (songs));
+	{
+		RBNodeView *view = rb_source_get_node_view (player->priv->selected_source);
+		rb_bonobo_set_sensitive (player->priv->component, CMD_PATH_SONG_INFO,
+					 rb_node_view_have_selection (view));
+	}
 
 	if (monkey_media_player_playing (player->priv->mmplayer)) {
 		if (player->priv->source == player->priv->selected_source
@@ -1251,18 +1263,31 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 	if (player->priv->source == source && source != NULL)
 		return;
 
-	/* Stop the already playing source. */
-	if (player->priv->source != NULL && sync_node_view) {
-		RBNodeView *songs = rb_source_get_node_view (player->priv->source);
-		rb_debug ("source is already playing, stopping it");
-		rb_node_view_set_playing_node (songs, NULL);
-		rb_node_view_set_playing (songs, FALSE);
-	}
-
 	rb_debug ("setting playing source to %p", source);
 
+	/* Stop the already playing source. */
+	if (player->priv->source != NULL) {
+		RBNodeView *songs = rb_source_get_node_view (player->priv->source);
+		if (sync_node_view) {
+			rb_debug ("source is already playing, stopping it");
+			rb_node_view_set_playing_node (songs, NULL);
+			rb_node_view_set_playing (songs, FALSE);
+		}
+		g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
+						      G_CALLBACK (rb_shell_player_playing_node_removed_cb),
+						      player);
+	}
+	
 	player->priv->source = source;
 
+	if (source != NULL) {
+		RBNodeView *songs = rb_source_get_node_view (player->priv->source);
+		g_signal_connect (G_OBJECT (songs),
+				  "playing_node_removed",
+				  G_CALLBACK (rb_shell_player_playing_node_removed_cb),
+				  player);
+	}
+
 	player->priv->song = NULL;
 	player->priv->url = NULL;
 	player->priv->have_url = FALSE;
@@ -1325,7 +1350,7 @@ eos_cb (MonkeyMediaPlayer *mmplayer, gpointer data)
  	RBShellPlayer *player = RB_SHELL_PLAYER (data);
 	rb_debug ("eos!");
 
-	gdk_threads_enter ();
+	GDK_THREADS_ENTER ();
 
 	if (player->priv->source != NULL)
 	{
@@ -1349,7 +1374,7 @@ eos_cb (MonkeyMediaPlayer *mmplayer, gpointer data)
 		}
 	}
 
-	gdk_threads_leave ();
+	GDK_THREADS_LEAVE ();
 }
 
 void
@@ -1651,7 +1676,7 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	XEvent *xev;
 	XKeyEvent *key;
 	RBShellPlayer *player;
-
+	RBSource *source; 
 	xev = (XEvent *) xevent;
 	if (xev->type != KeyPress) {
 		return GDK_FILTER_CONTINUE;
@@ -1661,6 +1686,9 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 
 	player = (RBShellPlayer *)data;
 
+	source = rb_shell_player_get_playing_node (player) == NULL ?
+			player->priv->selected_source : player->priv->source;
+
 	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {	
 		rb_shell_player_playpause (player);
 		return GDK_FILTER_REMOVE;
@@ -1668,10 +1696,12 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 		rb_shell_player_set_playing_source (player, NULL);
 		return GDK_FILTER_REMOVE;		
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
-		rb_shell_player_do_previous (player);
+		if (rb_shell_player_have_previous (player, source))
+			rb_shell_player_do_previous (player);
 		return GDK_FILTER_REMOVE;		
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
-		rb_shell_player_do_next (player);
+		if (rb_shell_player_have_next (player, source))
+			rb_shell_player_do_next (player);
 		return GDK_FILTER_REMOVE;
 	} else {
 		return GDK_FILTER_CONTINUE;
@@ -1701,7 +1731,8 @@ rb_shell_player_init_mmkeys (RBShellPlayer *shell_player)
 			root = gdk_screen_get_root_window (screen);
 
 			for (j = 0; j < 4 ; j++) {
-				grab_mmkey (keycodes[j], root);
+				if (keycodes[j] != 0)
+					grab_mmkey (keycodes[j], root);
 			}
 
 			gdk_window_add_filter (root, filter_mmkeys,
