@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-shell-preferences.c,v 1.31 2003/08/09 00:11:01 cwalters Exp $
+ *  $Id: rb-shell-preferences.c,v 1.33 2003/08/19 19:38:12 cwalters Exp $
  */
 
 #include <config.h>
@@ -133,10 +133,9 @@ help_cb (GtkWidget *widget,
 {
 	GError *error = NULL;
 
-	gnome_help_display ("net-rhythmbox.xml", "prefs", &error);
+	gnome_help_display ("rhythmbox.xml", "prefs", &error);
 
-	if (error != NULL)
-	{
+	if (error != NULL) {
 		rb_error_dialog (error->message);
 
 		g_error_free (error);
@@ -282,7 +281,7 @@ rb_shell_preferences_new (GList *views)
 	for (; views; views = views->next)
 	{
 		const char *name = NULL;
-		g_object_get (G_OBJECT (views->data), "config-name", &name, NULL);
+		g_object_get (G_OBJECT (views->data), "name", &name, NULL);
 		g_assert (name != NULL);
 		rb_shell_preferences_append_view_page (shell_preferences,
 						       name,
