@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-shell.c,v 1.161 2003/08/17 02:32:06 cwalters Exp $
+ *  $Id: rb-shell.c,v 1.175 2003/08/26 15:14:54 cwalters Exp $
  */
 
 #include <bonobo/bonobo-main.h>
@@ -72,7 +72,7 @@
 #include "rb-audiocd-source.h"
 #endif
 #include "rb-shell-preferences.h"
-#include "rb-group-source.h"
+#include "rb-playlist-source.h"
 #include "rb-file-monitor.h"
 #include "rb-library-dnd-types.h"
 #include "rb-volume.h"
@@ -119,6 +119,7 @@ static void source_selected_cb (RBSourceList *sourcelist,
 static void rb_shell_library_error_cb (RBLibrary *library,
 				       const char *uri, const char *msg,
 				       RBShell *shell); 
+static void rb_shell_library_progress_cb (RBLibrary *library, float progress, RBShell *shell); 
 static void rb_shell_load_failure_dialog_response_cb (GtkDialog *dialog,
 						      int response_id,
 						      RBShell *shell);
@@ -174,29 +175,23 @@ static void rb_shell_cmd_current_song (BonoboUIComponent *component,
 				       RBShell *shell,
 				       const char *verbname);
 static void rb_shell_jump_to_current (RBShell *shell);
-GtkWidget * rb_shell_new_group_dialog (RBShell *shell);
+GtkWidget * rb_shell_new_playlist_dialog (RBShell *shell);
+GtkWidget * rb_shell_rename_playlist_dialog (RBShell *shell);
 static void rb_shell_quit (RBShell *shell);
 static void rb_shell_view_sourcelist_changed_cb (BonoboUIComponent *component,
 						 const char *path,
 						 Bonobo_UIComponent_EventType type,
 						 const char *state,
 						 RBShell *shell);
-static void rb_shell_show_window_changed_cb (BonoboUIComponent *component,
-				             const char *path,
-				             Bonobo_UIComponent_EventType type,
-				             const char *state,
-				             RBShell *shell);
+static void rb_shell_cmd_show_window (BonoboUIComponent *component,
+				      RBShell *shell,
+				      const char *verbname);
 static void rb_shell_load_playlists (RBShell *shell);
 static void rb_shell_sync_sourcelist_visibility (RBShell *shell);
-static void rb_shell_sync_window_visibility (RBShell *shell);
 static void sourcelist_visibility_changed_cb (GConfClient *client,
 					      guint cnxn_id,
 					      GConfEntry *entry,
 					      RBShell *shell);
-static void window_visibility_changed_cb (GConfClient *client,
-			                  guint cnxn_id,
-			                  GConfEntry *entry,
-			                  RBShell *shell);
 static void paned_changed_cb (GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
@@ -214,7 +209,7 @@ static gboolean rb_shell_show_popup_cb (RBSourceList *sourcelist,
 					RBSource *target,
 					RBShell *shell);
 static void setup_tray_icon (RBShell *shell);
-static void sync_tray_menu (RBShell *shell);
+static void sync_tray_menu (RBShell *shell, guint button);
 
 static const GtkTargetEntry target_table[] =
 	{
@@ -229,17 +224,18 @@ static const GtkTargetEntry target_uri [] =
 
 typedef enum
 {
-	CREATE_GROUP_WITH_URI_LIST,
-	CREATE_GROUP_WITH_NODE_LIST,
-	CREATE_GROUP_WITH_FILE,
-	CREATE_GROUP_WITH_SELECTION
-} CreateGroupType;
+	CREATE_PLAYLIST_WITH_URI_LIST,
+	CREATE_PLAYLIST_WITH_NODE_LIST,
+	CREATE_PLAYLIST_WITH_FILE,
+	CREATE_PLAYLIST_WITH_SELECTION
+} CreatePlaylistType;
 
-static void create_group (RBShell *shell, CreateGroupType type, GList *data);
+static void create_playlist (RBShell *shell, CreatePlaylistType type, GList *data);
 
 #define CMD_PATH_VIEW_SOURCELIST   "/commands/ShowSourceList"
 #define CMD_PATH_SHOW_WINDOW    "/commands/ShowWindow"
 #define CMD_PATH_PLAYLIST_DELETE   "/commands/FileDeletePlaylist"
+#define CMD_PATH_PLAYLIST_RENAME   "/commands/FileRenamePlaylist"
 #define CMD_PATH_PLAYLIST_SAVE   "/commands/SavePlaylist"
 #define CMD_PATH_EXTRACT_CD     "/commands/ExtractCD"
 #define CMD_PATH_CURRENT_SONG	"/commands/CurrentSong"
@@ -248,7 +244,6 @@ static void create_group (RBShell *shell, CreateGroupType type, GList *data);
 #define CONF_STATE_WINDOW_WIDTH     CONF_PREFIX "/state/window_width"
 #define CONF_STATE_WINDOW_HEIGHT    CONF_PREFIX "/state/window_height"
 #define CONF_STATE_WINDOW_MAXIMIZED CONF_PREFIX "/state/window_maximized"
-#define CONF_STATE_WINDOW_HIDDEN    CONF_PREFIX "/state/window_hidden"
 #define CONF_STATE_PANED_POSITION   CONF_PREFIX "/state/paned_position"
 #define CONF_STATE_ADD_DIR          CONF_PREFIX "/state/add_dir"
 
@@ -287,7 +282,7 @@ struct RBShellPrivate
 
 	GtkWidget *prefs;
 
-	GList *groups;
+	GList *playlists;
 
 	EggTrayIcon *tray_icon;
 	GtkTooltips *tray_icon_tooltip;
@@ -298,7 +293,7 @@ struct RBShellPrivate
 
 	gboolean busy_cursor_displayed;
 
-	RBGroupSource *loading_group;
+	RBPlaylistSource *loading_playlist;
 };
 
 static BonoboUIVerb rb_shell_verbs[] =
@@ -314,8 +309,9 @@ static BonoboUIVerb rb_shell_verbs[] =
  	BONOBO_UI_VERB ("NewPlaylist",  (BonoboUIVerbFn) rb_shell_cmd_new_playlist),
 	BONOBO_UI_VERB ("NewStation",   (BonoboUIVerbFn) rb_shell_cmd_new_station),
 	BONOBO_UI_VERB ("FileDeletePlaylist",(BonoboUIVerbFn) rb_shell_cmd_delete_playlist),
-	BONOBO_UI_VERB ("RenamePlaylist",(BonoboUIVerbFn) rb_shell_cmd_rename_playlist),
 	BONOBO_UI_VERB ("DeletePlaylist",  (BonoboUIVerbFn) rb_shell_cmd_delete_playlist),
+	BONOBO_UI_VERB ("FileRenamePlaylist",(BonoboUIVerbFn) rb_shell_cmd_rename_playlist),
+	BONOBO_UI_VERB ("RenamePlaylist",(BonoboUIVerbFn) rb_shell_cmd_rename_playlist),
 	BONOBO_UI_VERB ("ExtractCD",  (BonoboUIVerbFn) rb_shell_cmd_extract_cd),
 	BONOBO_UI_VERB ("CurrentSong",	(BonoboUIVerbFn) rb_shell_cmd_current_song),
 	BONOBO_UI_VERB_END
@@ -327,10 +323,10 @@ static RBBonoboUIListener rb_shell_listeners[] =
 	RB_BONOBO_UI_LISTENER_END
 };
 
-static RBBonoboUIListener rb_tray_listeners[] =
+static BonoboUIVerb rb_tray_verbs[] =
 {
-	RB_BONOBO_UI_LISTENER ("ShowWindow", (BonoboUIListenerFn) rb_shell_show_window_changed_cb),
-	RB_BONOBO_UI_LISTENER_END
+	BONOBO_UI_VERB ("ShowWindow", (BonoboUIVerbFn) rb_shell_cmd_show_window),
+	BONOBO_UI_VERB_END
 };
 
 static GObjectClass *parent_class;
@@ -407,6 +403,7 @@ static void
 rb_shell_finalize (GObject *object)
 {
         RBShell *shell = RB_SHELL (object);
+	GList *tmp;
 
 	gtk_widget_hide (shell->priv->window);
 	gtk_widget_hide (GTK_WIDGET (shell->priv->tray_icon));
@@ -428,7 +425,11 @@ rb_shell_finalize (GObject *object)
 	
 	g_list_free (shell->priv->sources);
 
-	g_list_free (shell->priv->groups);
+	rb_debug ("saving playlists");
+	for (tmp = shell->priv->playlists; tmp; tmp = tmp->next)
+		rb_playlist_source_save (RB_PLAYLIST_SOURCE (tmp->data));
+
+	g_list_free (shell->priv->playlists);
 
 	g_object_unref (G_OBJECT (shell->priv->clipboard_shell));
 
@@ -532,7 +533,6 @@ rb_shell_corba_grab_focus (PortableServer_Servant _servant,
 			   CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
-	eel_gconf_set_boolean (CONF_STATE_WINDOW_HIDDEN, FALSE);
 
 	gtk_window_present (GTK_WINDOW (shell->priv->window));
 	gtk_widget_grab_focus (shell->priv->window);
@@ -648,9 +648,9 @@ rb_shell_construct (RBShell *shell)
 	rb_bonobo_add_listener_list_with_data (shell->priv->ui_component,
 					       rb_shell_listeners,
 					       shell);
-	rb_bonobo_add_listener_list_with_data (shell->priv->tray_icon_component,
-					       rb_tray_listeners,
-					       shell);
+	bonobo_ui_component_add_verb_list_with_data (shell->priv->tray_icon_component,
+						     rb_tray_verbs,
+						     shell);
 
 	/* initialize shell services */
 	rb_debug ("shell: initializing shell services");
@@ -671,7 +671,7 @@ rb_shell_construct (RBShell *shell)
 	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "show_popup",
 			  G_CALLBACK (rb_shell_show_popup_cb), shell);
 	
-	shell->priv->statusbar = rb_statusbar_new ();
+	shell->priv->statusbar = rb_statusbar_new (shell->priv->ui_component);
 
 	rb_sourcelist_set_dnd_targets (RB_SOURCELIST (shell->priv->sourcelist), target_table,
 				       G_N_ELEMENTS (target_table));
@@ -718,9 +718,6 @@ rb_shell_construct (RBShell *shell)
 	eel_gconf_notification_add (CONF_UI_SOURCELIST_HIDDEN,
 				    (GConfClientNotifyFunc) sourcelist_visibility_changed_cb,
 				    shell);
-	eel_gconf_notification_add (CONF_STATE_WINDOW_HIDDEN,
-				    (GConfClientNotifyFunc) window_visibility_changed_cb,
-				    shell);
 	eel_gconf_notification_add (CONF_STATE_PANED_POSITION,
 				    (GConfClientNotifyFunc) paned_changed_cb,
 				    shell);
@@ -736,6 +733,9 @@ rb_shell_construct (RBShell *shell)
 
 	g_signal_connect (G_OBJECT (shell->priv->library), "error",
 			  G_CALLBACK (rb_shell_library_error_cb), shell);
+	g_signal_connect (G_OBJECT (shell->priv->library), "progress",
+			  G_CALLBACK (rb_shell_library_progress_cb), shell);
+	
 	g_signal_connect (G_OBJECT (shell->priv->load_error_dialog), "response",
 			  G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell);
 
@@ -805,7 +805,7 @@ rb_shell_construct (RBShell *shell)
 		/* this is not critical, but worth a warning nevertheless */
 		char *msg = rb_shell_corba_exception_to_string (&ev);
 		g_message (_("Failed to register the shell: %s\n"
-			     "This probably means that you installed RB in a\n"
+			     "This probably means that you installed Rhythmbox in a\n"
 			     "different prefix than bonobo-activation; this\n"
 			     "warning is harmless, but IPC will not work.\n"), msg);
 		g_free (msg);
@@ -815,7 +815,7 @@ rb_shell_construct (RBShell *shell)
 
 	rb_debug ("Registered with Bonobo Activation");
 
-	/* now that the lib is loaded, we can load the music groups */
+	/* now that the lib is loaded, we can load the music playlists */
 	rb_debug ("shell: loading playlists");
 	rb_shell_load_playlists (shell);
 
@@ -831,8 +831,8 @@ rb_shell_construct (RBShell *shell)
 		rb_druid_show (druid);
 		g_object_unref (G_OBJECT (druid));
 	}
-	rb_shell_sync_window_visibility (shell);
 	
+	gtk_widget_show (GTK_WIDGET (shell->priv->window));
 	g_idle_add ((GSourceFunc) async_library_release_brakes, shell);
 }
 
@@ -919,6 +919,13 @@ source_selected_cb (RBSourceList *sourcelist,
 	rb_shell_select_source (shell, source);
 }
 
+static void
+rb_shell_library_progress_cb (RBLibrary *library, float progress, RBShell *shell)
+{
+	rb_debug ("setting progress to %f", progress);
+	rb_statusbar_set_progress (shell->priv->statusbar, progress);
+}
+
 static void
 rb_shell_library_error_cb (RBLibrary *library,
 			   const char *uri, const char *msg,
@@ -995,9 +1002,9 @@ rb_shell_remove_source (RBShell *shell,
 
 	shell->priv->sources = g_list_remove (shell->priv->sources, source);
 
-	if (g_list_find (shell->priv->groups, source) != NULL) {
-		shell->priv->groups = g_list_remove (shell->priv->groups, source);
-		rb_group_source_delete (RB_GROUP_SOURCE (source));
+	if (g_list_find (shell->priv->playlists, source) != NULL) {
+		shell->priv->playlists = g_list_remove (shell->priv->playlists, source);
+		rb_playlist_source_delete (RB_PLAYLIST_SOURCE (source));
 	}
 
 	rb_sourcelist_remove (RB_SOURCELIST (shell->priv->sourcelist), source);
@@ -1035,10 +1042,13 @@ rb_shell_select_source (RBShell *shell,
 	rb_statusbar_set_source (shell->priv->statusbar,
 				 RB_SOURCE (source));
 	rb_bonobo_set_sensitive (shell->priv->ui_component, CMD_PATH_PLAYLIST_DELETE,
-				 g_list_find (shell->priv->groups,
+				 g_list_find (shell->priv->playlists,
+					      shell->priv->selected_source) != NULL);
+	rb_bonobo_set_sensitive (shell->priv->ui_component, CMD_PATH_PLAYLIST_RENAME,
+				 g_list_find (shell->priv->playlists,
 					      shell->priv->selected_source) != NULL);
 	rb_bonobo_set_sensitive (shell->priv->ui_component, CMD_PATH_PLAYLIST_SAVE,
-				 g_list_find (shell->priv->groups,
+				 g_list_find (shell->priv->playlists,
 					      shell->priv->selected_source) != NULL);
 }
 
@@ -1113,14 +1123,13 @@ rb_shell_view_sourcelist_changed_cb (BonoboUIComponent *component,
 }
 
 static void
-rb_shell_show_window_changed_cb (BonoboUIComponent *component,
-				 const char *path,
-				 Bonobo_UIComponent_EventType type,
-				 const char *state,
-				 RBShell *shell)
+rb_shell_cmd_show_window (BonoboUIComponent *component,
+			  RBShell *shell,
+			  const char *verbname)
 {
-	eel_gconf_set_boolean (CONF_STATE_WINDOW_HIDDEN,
-			       !rb_bonobo_get_active (component, CMD_PATH_SHOW_WINDOW));
+	rb_debug ("showing window");
+	gtk_window_present (GTK_WINDOW (shell->priv->window));
+	gtk_widget_grab_focus (shell->priv->window);
 }
 
 static void
@@ -1175,10 +1184,10 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 		backend = "Xine";
 #endif		
 #ifdef HAVE_MP3
-		g_string_append (formats, "mp3 ");
+		g_string_append (formats, "MP3 ");
 #endif
 #ifdef HAVE_VORBIS
-		g_string_append (formats, "vorbis ");
+		g_string_append (formats, "Vorbis ");
 #endif
 #ifdef HAVE_FLAC
 		g_string_append (formats, "FLAC ");
@@ -1313,7 +1322,7 @@ load_playlist_response_cb (GtkDialog *dialog,
 	shell->priv->show_library_errors = TRUE;
 
 	tem = g_list_append (NULL, file);
-	create_group (shell, CREATE_GROUP_WITH_FILE, tem);
+	create_playlist (shell, CREATE_PLAYLIST_WITH_FILE, tem);
 
 	shell->priv->show_library_errors = FALSE;
 }
@@ -1385,10 +1394,51 @@ save_playlist_response_cb (GtkDialog *dialog,
 	if (file == NULL)
 		return;
 
-	rb_group_source_save_playlist (RB_GROUP_SOURCE (shell->priv->selected_source), file);
+	rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (shell->priv->selected_source), file);
 	g_free (file);
 }
 
+static void
+ask_rename_response_cb (GtkDialog *dialog,
+			int response_id,
+			RBShell *shell)
+{
+	GtkWidget *entry;
+	char *name;
+
+	rb_debug ("rename response");
+
+	if (response_id != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		return;
+	}
+
+	entry = g_object_get_data (G_OBJECT (dialog), "entry");
+	name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	if (name == NULL) {  /* user didn't put anything */
+		return;
+	} else {
+		RBPlaylistSource *ret;
+		GList *tem;
+		char *temname;
+		
+		for (tem = shell->priv->playlists; tem; tem = g_list_next (tem)) {
+			g_object_get (G_OBJECT (tem->data), "name", &temname, NULL);
+			if (!strcmp (temname, name)) {
+				rb_error_dialog (_("There is already a playlist with that name."));
+				return;
+			}
+		}
+		ret = RB_PLAYLIST_SOURCE (shell->priv->selected_source);
+		g_object_set (G_OBJECT (ret), "name", name, NULL);
+	}
+
+	g_free(name);
+}
+
 static void
 rb_shell_cmd_save_playlist (BonoboUIComponent *component,
 			    RBShell *shell,
@@ -1406,29 +1456,29 @@ rb_shell_cmd_save_playlist (BonoboUIComponent *component,
 			  shell);
 }
 
-static RBGroupSource *
-create_group_with_name (RBShell *shell, const char *name)
+static RBPlaylistSource *
+create_playlist_with_name (RBShell *shell, const char *name)
 {
-	RBGroupSource *ret;
+	RBPlaylistSource *ret;
 	GList *tem;
 	char *temname;
 
-	for (tem = shell->priv->groups; tem; tem = g_list_next (tem)) {
+	for (tem = shell->priv->playlists; tem; tem = g_list_next (tem)) {
 		g_object_get (G_OBJECT (tem->data), "name", &temname, NULL);
 		if (!strcmp (temname, name)) {
 			rb_error_dialog (_("There is already a playlist with that name."));
 			return NULL;
 		}
 	}
-	ret = RB_GROUP_SOURCE (rb_group_source_new (shell->priv->container,
-						    shell->priv->library,
-						    RB_LIBRARY_SOURCE (shell->priv->library_source)));
-	rb_group_source_set_name (RB_GROUP_SOURCE (ret), name);
+	ret = RB_PLAYLIST_SOURCE (rb_playlist_source_new (shell->priv->container,
+							  shell->priv->library,
+							  RB_LIBRARY_SOURCE (shell->priv->library_source)));
+	g_object_set (G_OBJECT (ret), "name", name, NULL);
 	return ret;
 }
 
 static void
-add_uri_to_group (RBShell *shell, RBGroupSource *group, const char *uri, const char *title)
+add_uri_to_playlist (RBShell *shell, RBPlaylistSource *playlist, const char *uri, const char *title)
 {
 	RBNode *node;
 	GError *error = NULL;
@@ -1450,17 +1500,17 @@ add_uri_to_group (RBShell *shell, RBGroupSource *group, const char *uri, const c
 
 	g_return_if_fail (node != NULL);
 
-	/* add this node to the newly created group */
-	rb_group_source_add_node (group, node);
+	/* add this node to the newly created playlist */
+	rb_playlist_source_add_node (playlist, node);
 out:
 	gnome_vfs_uri_unref (vfsuri);
 }
 
 static void
-handle_playlist_entry_into_group_cb (RBPlaylist *playlist, const char *uri, const char *title,
-				     const char *genre, RBShell *shell)
+handle_playlist_entry_into_playlist_cb (RBPlaylist *playlist, const char *uri, const char *title,
+					const char *genre, RBShell *shell)
 {
-	add_uri_to_group (shell, RB_GROUP_SOURCE (shell->priv->loading_group), uri, title);
+	add_uri_to_playlist (shell, RB_PLAYLIST_SOURCE (shell->priv->loading_playlist), uri, title);
 }
 
 static void
@@ -1469,10 +1519,10 @@ ask_string_response_cb (GtkDialog *dialog,
 			RBShell *shell)
 {
 	GtkWidget *entry, *checkbox;
-	RBSource *group;
+	RBSource *playlist;
 	char *name;
 	gboolean add_selection;
-	CreateGroupType type;
+	CreatePlaylistType type;
 	GList *data, *l;
 
 	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "type"));
@@ -1480,7 +1530,7 @@ ask_string_response_cb (GtkDialog *dialog,
 
 	if (response_id != GTK_RESPONSE_OK) {
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		if (type == CREATE_GROUP_WITH_URI_LIST)
+		if (type == CREATE_PLAYLIST_WITH_URI_LIST)
 			gnome_vfs_uri_list_free (data);
 		return;
 	}
@@ -1494,70 +1544,70 @@ ask_string_response_cb (GtkDialog *dialog,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	
 	if (name == NULL) {
-		if (type == CREATE_GROUP_WITH_URI_LIST)
+		if (type == CREATE_PLAYLIST_WITH_URI_LIST)
 			gnome_vfs_uri_list_free (data);
 		return;
 	}
 
-	group = RB_SOURCE (create_group_with_name (shell, name));
-	if (group == NULL)
+	playlist = RB_SOURCE (create_playlist_with_name (shell, name));
+	if (playlist == NULL)
 		return;
 
-	shell->priv->groups = g_list_append (shell->priv->groups, group);
-	rb_shell_append_source (shell, group);
+	shell->priv->playlists = g_list_append (shell->priv->playlists, playlist);
+	rb_shell_append_source (shell, playlist);
 	g_free (name);
 
 	switch (type)
 	{
-	case CREATE_GROUP_WITH_NODE_LIST:
+	case CREATE_PLAYLIST_WITH_NODE_LIST:
 		for (l = data; l != NULL; l = g_list_next (l))
-			rb_group_source_add_node (RB_GROUP_SOURCE (group),
+			rb_playlist_source_add_node (RB_PLAYLIST_SOURCE (playlist),
 						  l->data);
 		break;
-	case CREATE_GROUP_WITH_FILE:
+	case CREATE_PLAYLIST_WITH_FILE:
 	{
 		RBPlaylist *parser = rb_playlist_new ();
 		g_signal_connect (G_OBJECT (parser), "entry",
-				  G_CALLBACK (handle_playlist_entry_into_group_cb),
+				  G_CALLBACK (handle_playlist_entry_into_playlist_cb),
 				  shell);
 		
-		shell->priv->loading_group = RB_GROUP_SOURCE (group);
+		shell->priv->loading_playlist = RB_PLAYLIST_SOURCE (playlist);
 		if (!rb_playlist_parse (parser, g_list_first (data)->data))
 			rb_error_dialog (_("Couldn't parse playlist"));
-		shell->priv->loading_group = NULL;
+		shell->priv->loading_playlist = NULL;
 		g_free (g_list_first (data)->data);
 		g_list_free (data);
 		g_object_unref (G_OBJECT (parser));
 	}
 	break;
-	case CREATE_GROUP_WITH_URI_LIST:
+	case CREATE_PLAYLIST_WITH_URI_LIST:
 		for (l = data; l != NULL; l = g_list_next (l)) {
 			char *uri;
 			uri = gnome_vfs_uri_to_string ((GnomeVFSURI *) l->data, GNOME_VFS_URI_HIDE_NONE);
-			add_uri_to_group (shell, RB_GROUP_SOURCE (group), uri, NULL);
+			add_uri_to_playlist (shell, RB_PLAYLIST_SOURCE (playlist), uri, NULL);
 			g_free (uri);
 		}
 		gnome_vfs_uri_list_free (data);
 		break;
-	case CREATE_GROUP_WITH_SELECTION:
+	case CREATE_PLAYLIST_WITH_SELECTION:
 		/* add the current selection if the user checked */
 		if (add_selection) {
 			RBNodeView *nodeview = rb_source_get_node_view (shell->priv->selected_source);
 			GList *i = NULL;
 			GList *selection = rb_node_view_get_selection (nodeview);
 			for (i  = selection; i != NULL; i = g_list_next (i))
-				rb_group_source_add_node (RB_GROUP_SOURCE (group), i->data);
+				rb_playlist_source_add_node (RB_PLAYLIST_SOURCE (playlist), i->data);
 		}
 	break;
 	}
 }
 
 static void
-create_group (RBShell *shell, CreateGroupType type, GList *data)
+create_playlist (RBShell *shell, CreatePlaylistType type, GList *data)
 {
 	GtkWidget *dialog;
 	
-	dialog = rb_shell_new_group_dialog (shell);
+	dialog = rb_shell_new_playlist_dialog (shell);
 
 	g_object_set_data (G_OBJECT (dialog), "type", GINT_TO_POINTER (type));
 	g_object_set_data (G_OBJECT (dialog), "data", data);
@@ -1573,7 +1623,7 @@ rb_shell_cmd_new_playlist (BonoboUIComponent *component,
 			RBShell *shell,
 			const char *verbname)
 {
-	create_group (shell, CREATE_GROUP_WITH_SELECTION, NULL);
+	create_playlist (shell, CREATE_PLAYLIST_WITH_SELECTION, NULL);
 }
 
 static void
@@ -1581,7 +1631,14 @@ rb_shell_cmd_rename_playlist (BonoboUIComponent *component,
 			      RBShell *shell,
 			      const char *verbname)
 {
-	rb_debug ("FIXME");
+	GtkWidget *dialog;
+	
+	dialog = rb_shell_rename_playlist_dialog (shell);
+	
+	g_signal_connect (G_OBJECT (dialog),
+			  "response",
+			  G_CALLBACK (ask_rename_response_cb),
+			  shell);
 }
 
 static void
@@ -1628,12 +1685,22 @@ rb_shell_quit (RBShell *shell)
 static void
 rb_shell_load_playlists (RBShell *shell)
 {
-	char *path;
+	char *oldpath, *path;
 	GnomeVFSDirectoryHandle *handle;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 
-	path = g_build_filename (rb_dot_dir (), "groups", NULL);
+	/* Backwards compatibility with old "group" name */
+	oldpath = g_build_filename (rb_dot_dir (), "groups", NULL);
+	path = g_build_filename (rb_dot_dir (), "playlists", NULL);
+
+	if ((result = gnome_vfs_directory_open (&handle, oldpath, GNOME_VFS_FILE_INFO_FOLLOW_LINKS))
+	    == GNOME_VFS_OK) {
+		rb_debug ("Renaming legacy group dir");
+		gnome_vfs_move (oldpath, path, TRUE);
+	}
+		
+	g_free (oldpath);
 
 	if ((result = gnome_vfs_directory_open (&handle, path, GNOME_VFS_FILE_INFO_FOLLOW_LINKS))
 	    != GNOME_VFS_OK)
@@ -1641,7 +1708,7 @@ rb_shell_load_playlists (RBShell *shell)
 
 	info = gnome_vfs_file_info_new ();
 	while ((result = gnome_vfs_directory_read_next (handle, info)) == GNOME_VFS_OK) {
-		RBSource *group;
+		RBSource *playlist;
 		char *filepath;
 
 		if (info->name[0] == '.')
@@ -1649,14 +1716,14 @@ rb_shell_load_playlists (RBShell *shell)
 
 		filepath = g_build_filename (path, info->name, NULL);
 
-		group = rb_group_source_new_from_file (shell->priv->container,
-						       shell->priv->library,
-						       RB_LIBRARY_SOURCE (shell->priv->library_source),
-						       filepath);
-		if (group != NULL) {
-			shell->priv->groups = g_list_append (shell->priv->groups, group);
+		playlist = rb_playlist_source_new_from_file (shell->priv->container,
+							     shell->priv->library,
+							     RB_LIBRARY_SOURCE (shell->priv->library_source),
+							     filepath);
+		if (playlist != NULL) {
+			shell->priv->playlists = g_list_append (shell->priv->playlists, playlist);
 			
-			rb_shell_append_source (shell, group);
+			rb_shell_append_source (shell, playlist);
 		}
 		g_free (filepath);
 	}
@@ -1683,33 +1750,6 @@ rb_shell_sync_sourcelist_visibility (RBShell *shell)
 			      visible);
 }
 
-static void
-rb_shell_sync_window_visibility (RBShell *shell)
-{
-	gboolean visible;
-	static int window_x = -1;
-	static int window_y = -1;
-
-	rb_debug ("syncing visibility");
-	visible = !eel_gconf_get_boolean (CONF_STATE_WINDOW_HIDDEN);
-	
-	if (visible == TRUE) {
-		if (window_x >= 0 && window_y >= 0) {
-			gtk_window_move (GTK_WINDOW (shell->priv->window), window_x,
-					 window_y);
-		}
-		gtk_widget_show (shell->priv->window);
-	} else {
-		gtk_window_get_position (GTK_WINDOW (shell->priv->window),
-					 &window_x, &window_y);
-		gtk_widget_hide (shell->priv->window);
-	}
-	
-	rb_bonobo_set_active (shell->priv->ui_component,
-			      CMD_PATH_SHOW_WINDOW,
-			      visible);
-}
-
 static void
 sourcelist_visibility_changed_cb (GConfClient *client,
 				  guint cnxn_id,
@@ -1720,16 +1760,6 @@ sourcelist_visibility_changed_cb (GConfClient *client,
 	rb_shell_sync_sourcelist_visibility (shell);
 }
 
-static void
-window_visibility_changed_cb (GConfClient *client,
-			      guint cnxn_id,
-			      GConfEntry *entry,
-			      RBShell *shell)
-{
-	rb_debug ("window visiblity changed");
-	rb_shell_sync_window_visibility (shell);
-}
-
 static void
 rb_shell_sync_paned (RBShell *shell)
 {
@@ -1787,9 +1817,9 @@ paned_changed_cb (GConfClient *client,
 
 static void
 handle_songs_func (RBNode *node,
-		   RBGroupSource *group)
+		   RBPlaylistSource *playlist)
 {
-	rb_group_source_add_node (group, node);
+	rb_playlist_source_add_node (playlist, node);
 }
 
 static void
@@ -1806,7 +1836,7 @@ sourcelist_drag_received_cb (RBSourceList *sourcelist,
 	if (data->type == gdk_atom_intern (RB_LIBRARY_DND_NODE_ID_TYPE, TRUE)) {
 		long id;
 		RBNode *node;
-		RBGroupSource *group;
+		RBPlaylistSource *playlist;
 
 
 		id = atol (data->data);
@@ -1817,23 +1847,23 @@ sourcelist_drag_received_cb (RBSourceList *sourcelist,
 			return;
 			
 		
-		group = create_group_with_name (shell, rb_node_get_property_string (node, RB_NODE_PROP_NAME));
-		if (group == NULL)
+		playlist = create_playlist_with_name (shell, rb_node_get_property_string (node, RB_NODE_PROP_NAME));
+		if (playlist == NULL)
 			return;
 
 		rb_library_handle_songs (shell->priv->library,
 					 node,
 					 (GFunc) handle_songs_func,
-					 group);
+					 playlist);
 
-		shell->priv->groups = g_list_append (shell->priv->groups, group);
-		rb_shell_append_source (shell, RB_SOURCE (group));
+		shell->priv->playlists = g_list_append (shell->priv->playlists, playlist);
+		rb_shell_append_source (shell, RB_SOURCE (playlist));
 	} else {
 		GList *list;
 
 		rb_debug ("got vfs data, len: %d", data->length);
 		list = gnome_vfs_uri_list_parse (data->data);
-		create_group (shell, CREATE_GROUP_WITH_URI_LIST, list);
+		create_playlist (shell, CREATE_PLAYLIST_WITH_URI_LIST, list);
 	}
 }
 
@@ -1872,14 +1902,14 @@ rb_shell_jump_to_current (RBShell *shell)
 	rb_shell_player_jump_to_current (shell->priv->player_shell);
 }
 
-/* rb_shell_new_group_dialog: create a dialog for creating a new
- * group.
+/* rb_shell_new_playlist_dialog: create a dialog for creating a new
+ * playlist.
  *
  * TODO Make this a gobject that could hold more functionality
  * like multi criteria search.
  */
 GtkWidget *
-rb_shell_new_group_dialog (RBShell *shell)
+rb_shell_new_playlist_dialog (RBShell *shell)
 {
 	RBNodeView *nodeview;
 	GtkWidget *dialog, *hbox, *image, *entry, *label, *vbox, *cbox, *align;
@@ -1909,7 +1939,7 @@ rb_shell_new_group_dialog (RBShell *shell)
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-	image = gtk_image_new_from_stock (RB_STOCK_GROUP,
+	image = gtk_image_new_from_stock (RB_STOCK_PLAYLIST,
 					  GTK_ICON_SIZE_DIALOG);
 	align = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
 	gtk_container_add (GTK_CONTAINER (align), image);
@@ -1946,6 +1976,64 @@ rb_shell_new_group_dialog (RBShell *shell)
 	return dialog;
 }
 
+GtkWidget *
+rb_shell_rename_playlist_dialog (RBShell *shell)
+{
+      GtkWidget *dialog, *hbox, *image, *entry, *label, *vbox, *align;
+
+      dialog = gtk_dialog_new_with_buttons ("",
+                                           NULL,
+                                           0,
+                                           GTK_STOCK_CANCEL,
+                                           GTK_RESPONSE_CANCEL,
+                                           _("R_ename"),
+                                           GTK_RESPONSE_OK,
+                                           NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+                                       GTK_RESPONSE_OK);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+      gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+      gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+
+      gtk_window_set_transient_for (GTK_WINDOW (dialog),
+                                    GTK_WINDOW (shell->priv->window));
+      gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
+      gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+      gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+      gtk_window_set_title (GTK_WINDOW (dialog), _("Rename Playlist"));
+
+      hbox = gtk_hbox_new (FALSE, 12);
+      gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+      image = gtk_image_new_from_stock (RB_STOCK_PLAYLIST,
+                                        GTK_ICON_SIZE_DIALOG);
+      align = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
+      gtk_container_add (GTK_CONTAINER (align), image);
+      gtk_box_pack_start (GTK_BOX (hbox), align, TRUE, TRUE, 0);
+      vbox = gtk_vbox_new (FALSE, 6);
+
+      label = gtk_label_new (_("Please enter a new name for the playlist:"));
+      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
+
+      entry = gtk_entry_new ();
+      gtk_entry_set_text (GTK_ENTRY (entry), _("Untitled"));
+      gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+      gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+      gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+
+      gtk_widget_show_all (hbox);
+      gtk_widget_show_all (vbox);
+      gtk_widget_grab_focus (entry);
+
+      g_object_set_data (G_OBJECT (dialog), "entry", entry);
+
+      gtk_widget_show_all (dialog);
+
+      return dialog;
+}
+
 static gboolean
 rb_shell_show_popup_cb (RBSourceList *sourcelist,
 			RBSource *target,
@@ -1961,21 +2049,11 @@ tray_button_press_event_cb (GtkWidget *ebox,
 			    GdkEventButton *event,
 			    RBShell *shell)
 {
-	switch (event->button)
-	{
-	case 1:
-		/* toggle mainwindow visibility */
-		eel_gconf_set_boolean (CONF_STATE_WINDOW_HIDDEN,
-				       !eel_gconf_get_boolean (CONF_STATE_WINDOW_HIDDEN));
-		break;
-	case 3:
-		/* contextmenu */
-		sync_tray_menu (shell);
-		bonobo_control_do_popup (shell->priv->tray_icon_control,
-					 event->button,
-					 event->time);
-		break;
-	}
+	/* contextmenu */
+	sync_tray_menu (shell, event->button);
+	bonobo_control_do_popup (shell->priv->tray_icon_control,
+				 event->button,
+				 event->time);
 }
 
 static void
@@ -2086,7 +2164,7 @@ setup_tray_icon (RBShell *shell)
 }
 
 static void
-sync_tray_menu (RBShell *shell)
+sync_tray_menu (RBShell *shell, guint button)
 {
 	BonoboUIComponent *pcomp;
 	BonoboUINode *node;
@@ -2096,7 +2174,20 @@ sync_tray_menu (RBShell *shell)
 	bonobo_ui_component_set (pcomp, "/", "<popups></popups>", NULL);
 
 	node = bonobo_ui_component_get_tree (shell->priv->ui_component, "/popups/TrayPopup", TRUE, NULL);
-	bonobo_ui_node_set_attr (node, "name", "button3");
+
+	switch (button) {
+	case 1:
+		bonobo_ui_node_set_attr (node, "name", "button1");
+		break;
+	case 2:
+		bonobo_ui_node_set_attr (node, "name", "button2");
+		break;
+	case 3:
+		bonobo_ui_node_set_attr (node, "name", "button3");
+		break;
+	default:
+		break;
+	}
 	bonobo_ui_component_set_tree (pcomp, "/popups", node, NULL);
 	bonobo_ui_node_free (node);
 
