@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-statusbar.c,v 1.6 2003/08/08 06:46:06 cwalters Exp $
+ *  $Id: rb-statusbar.c,v 1.8 2003/08/19 02:56:40 cwalters Exp $
  */
 
 #include <gtk/gtk.h>
@@ -25,9 +25,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-ui-util.h>
 
 #include "rb-statusbar.h"
 #include "rb-thread-helpers.h"
+#include "rb-bonobo-helpers.h"
 #include "rb-preferences.h"
 #include "rb-search-entry.h"
 #include "rb-debug.h"
@@ -47,7 +49,7 @@ static void rb_statusbar_get_property (GObject *object,
 static void rb_statusbar_sync_with_source (RBStatusbar *statusbar);
 static void rb_statusbar_status_changed_cb (RBSource *source,
 					    RBStatusbar *statusbar);
-static void rb_statusbar_sync_toggles (RBStatusbar *statusbar);
+static void rb_statusbar_sync_state (RBStatusbar *statusbar);
 static void rb_statusbar_state_changed_cb (GConfClient *client,
 					   guint cnxn_id,
 					   GConfEntry *entry,
@@ -55,19 +57,41 @@ static void rb_statusbar_state_changed_cb (GConfClient *client,
 
 static void rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
 					    RBStatusbar *statusbar);
+static void rb_statusbar_view_statusbar_changed_cb (BonoboUIComponent *component,
+						    const char *path,
+						    Bonobo_UIComponent_EventType type,
+						    const char *state,
+						    RBStatusbar *statusbar);
+
+#define CMD_PATH_VIEW_STATUSBAR	"/commands/ViewStatusbar"
+
+static RBBonoboUIListener rb_statusbar_listeners[] =
+{
+	RB_BONOBO_UI_LISTENER ("ViewStatusbar", (BonoboUIListenerFn) rb_statusbar_view_statusbar_changed_cb),
+	RB_BONOBO_UI_LISTENER_END
+};
+
 
 struct RBStatusbarPrivate
 {
 	RBSource *selected_source;
 
+	BonoboUIComponent *component;
+
 	GtkWidget *shuffle;
 	GtkWidget *repeat;
 	GtkWidget *status;
+
+	GtkWidget *progress;
+
+	gboolean idle_tick_running;
+	guint idle_tick_id;
 };
 
 enum
 {
 	PROP_0,
+	PROP_COMPONENT,
 	PROP_SOURCE,
 };
 
@@ -120,6 +144,13 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
 							      "RBSource object",
 							      RB_TYPE_SOURCE,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_COMPONENT,
+					 g_param_spec_object ("component",
+							      "BonoboUIComponent",
+							      "BonoboUIComponent object",
+							      BONOBO_TYPE_UI_COMPONENT,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -140,16 +171,24 @@ rb_statusbar_init (RBStatusbar *statusbar)
 
 	gtk_box_set_spacing (GTK_BOX (statusbar), 5);
 
+	statusbar->priv->progress = gtk_progress_bar_new ();
+
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->priv->progress), 1.0);
+
 	gtk_box_pack_start (GTK_BOX (statusbar),
 			    GTK_WIDGET (statusbar->priv->shuffle), FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (statusbar),
 			    GTK_WIDGET (statusbar->priv->repeat), FALSE, TRUE, 0);
 
+	gtk_box_pack_start (GTK_BOX (statusbar),
+			    GTK_WIDGET (statusbar->priv->status), TRUE, TRUE, 0);
+
 	gtk_box_pack_end (GTK_BOX (statusbar),
-			  GTK_WIDGET (statusbar->priv->status), TRUE, TRUE, 0);
-
-	rb_statusbar_sync_toggles (statusbar);
+			  GTK_WIDGET (statusbar->priv->progress), FALSE, TRUE, 0);
 
+	eel_gconf_notification_add (CONF_UI_STATUSBAR_HIDDEN,
+				    (GConfClientNotifyFunc) rb_statusbar_state_changed_cb,
+				    statusbar);
 	eel_gconf_notification_add (CONF_STATE_SHUFFLE,
 				    (GConfClientNotifyFunc) rb_statusbar_state_changed_cb,
 				    statusbar);
@@ -170,6 +209,10 @@ rb_statusbar_finalize (GObject *object)
 
 	g_return_if_fail (statusbar->priv != NULL);
 
+	if (statusbar->priv->idle_tick_running) {
+		g_source_remove (statusbar->priv->idle_tick_id);
+	}
+
 	g_free (statusbar->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -204,7 +247,15 @@ rb_statusbar_set_property (GObject *object,
 					  statusbar);
 		}
 		rb_statusbar_sync_with_source (statusbar);
-		
+
+		break;
+	case PROP_COMPONENT:
+		statusbar->priv->component = g_value_get_object (value);
+		rb_bonobo_add_listener_list_with_data (statusbar->priv->component,
+						       rb_statusbar_listeners,
+						       statusbar);
+
+		rb_statusbar_sync_state (statusbar);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -225,6 +276,9 @@ rb_statusbar_get_property (GObject *object,
 	case PROP_SOURCE:
 		g_value_set_object (value, statusbar->priv->selected_source);
 		break;
+	case PROP_COMPONENT:
+		g_value_set_object (value, statusbar->priv->component);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -243,10 +297,46 @@ rb_statusbar_set_source (RBStatusbar *statusbar,
 		      NULL);
 }
 
+static gboolean
+status_tick_cb (GtkProgressBar *progress)
+{
+	g_return_val_if_fail (GTK_IS_PROGRESS_BAR (progress), FALSE);
+
+	gdk_threads_enter ();
+
+	gtk_progress_bar_pulse (progress);
+
+	gdk_threads_leave ();
+
+	return TRUE;
+}
+
+void
+rb_statusbar_set_progress (RBStatusbar *statusbar, float progress)
+{
+	if (progress > 1.0)
+		progress = 1.0;
+
+	if (progress >= 1.0 && statusbar->priv->idle_tick_running) {
+		statusbar->priv->idle_tick_running = FALSE;		
+		g_source_remove (statusbar->priv->idle_tick_id);
+	}
+	
+	if (progress > 0.0)
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->priv->progress), progress);
+	else if (!statusbar->priv->idle_tick_running) {
+		statusbar->priv->idle_tick_running = TRUE;
+		statusbar->priv->idle_tick_id
+			= g_timeout_add (250, (GSourceFunc) status_tick_cb, statusbar->priv->progress);
+	}
+}
+
+
 RBStatusbar *
-rb_statusbar_new (void)
+rb_statusbar_new (BonoboUIComponent *component)
 {
 	RBStatusbar *statusbar = g_object_new (RB_TYPE_STATUSBAR,
+					       "component", component,
 					       NULL);
 
 	g_return_val_if_fail (statusbar->priv != NULL, NULL);
@@ -274,13 +364,23 @@ rb_statusbar_status_changed_cb (RBSource *source,
 }
 
 static void
-rb_statusbar_sync_toggles (RBStatusbar *statusbar)
+rb_statusbar_sync_state (RBStatusbar *statusbar)
 {
-	rb_debug ("syncing toggles");
+	gboolean hidden;
+	rb_debug ("syncing state");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
 				      eel_gconf_get_boolean (CONF_STATE_SHUFFLE));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->repeat),
 				      eel_gconf_get_boolean (CONF_STATE_REPEAT));
+	
+	hidden = eel_gconf_get_boolean (CONF_UI_STATUSBAR_HIDDEN);
+	if (hidden)
+		gtk_widget_hide (GTK_WIDGET (statusbar));
+	else
+		gtk_widget_show (GTK_WIDGET (statusbar));
+	rb_bonobo_set_active (statusbar->priv->component,
+			      CMD_PATH_VIEW_STATUSBAR,
+			      !hidden);
 }
 
 static void
@@ -291,7 +391,7 @@ rb_statusbar_state_changed_cb (GConfClient *client,
 {
 	rb_debug ("state changed");
 	
-	rb_statusbar_sync_toggles (statusbar);
+	rb_statusbar_sync_state (statusbar);
 }
 
 static void
@@ -304,3 +404,16 @@ rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
 	eel_gconf_set_boolean (CONF_STATE_REPEAT,
 			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (statusbar->priv->repeat)));
 }
+
+static void
+rb_statusbar_view_statusbar_changed_cb (BonoboUIComponent *component,
+					const char *path,
+					Bonobo_UIComponent_EventType type,
+					const char *state,
+					RBStatusbar *statusbar)
+{
+	rb_debug ("got view statusbar toggle");
+	eel_gconf_set_boolean (CONF_UI_STATUSBAR_HIDDEN,
+			       !rb_bonobo_get_active (component, CMD_PATH_VIEW_STATUSBAR));
+}
+
