@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-iradio-source.c,v 1.17 2003/08/12 22:08:04 cwalters Exp $
+ *  $Id: rb-iradio-source.c,v 1.18 2003/08/17 07:34:19 cwalters Exp $
  */
 
 #include <config.h>
@@ -78,7 +78,6 @@ static void rb_iradio_source_state_prefs_sync (RBIRadioSource *source);
 /* source methods */
 static const char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
-static const char *impl_get_description (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBNodeView *impl_get_node_view (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
@@ -198,7 +197,6 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 
 	source_class->impl_get_status  = impl_get_status;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
-	source_class->impl_get_description  = impl_get_description;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -358,9 +356,7 @@ rb_iradio_source_new (BonoboUIContainer *container,
 	RBSource *source;
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_IRADIO_SOURCE,
-					  "ui-file", "rhythmbox-iradio-view.xml",
-					  "ui-name", "IRadioSource",
-					  "config-name", "Internet Radio",
+					  "name", _("Radio"),
 					  "container", container,
 					  "backend", backend,
 					  NULL));
@@ -368,12 +364,6 @@ rb_iradio_source_new (BonoboUIContainer *container,
 	return source;
 }
 
-static const char *
-impl_get_description (RBSource *source)
-{
-	return _("Radio");
-}
-
 static GdkPixbuf *
 impl_get_pixbuf (RBSource *asource)
 {
