@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-library-source.c,v 1.27 2003/08/14 07:27:21 cwalters Exp $
+ *  $Id: rb-library-source.c,v 1.31 2003/08/19 21:56:07 cwalters Exp $
  */
 
 #include <config.h>
@@ -98,7 +98,6 @@ static void rb_library_source_preferences_sync (RBLibrarySource *source);
 /* source methods */
 static const char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
-static const char *impl_get_description (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBNodeView *impl_get_node_view (RBSource *source);
 static GList *impl_get_extra_views (RBSource *source);
@@ -150,6 +149,8 @@ struct RBLibrarySourcePrivate
 
 	gboolean lock;
 
+	char *status;
+	
 	RBNodeFilter *artists_filter;
 	RBNodeFilter *songs_filter;
 	RBNodeFilter *albums_filter;
@@ -224,7 +225,6 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	source_class->impl_get_status = impl_get_status;
 	source_class->impl_get_browser_key = impl_get_browser_key;
-	source_class->impl_get_description  = impl_get_description;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -353,7 +353,6 @@ rb_library_source_finalize (GObject *object)
 
 static void
 rb_library_source_songs_show_popup_cb (RBNodeView *view,
-				       RBNode *node,
 				       RBLibrarySource *library_source)
 {
 	GtkWidget *menu;
@@ -527,9 +526,7 @@ rb_library_source_new (BonoboUIContainer *container,
 	RBSource *source;
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
-					  "ui-file", "rhythmbox-library-view.xml",
-					  "ui-name", "LibraryView",
-					  "config-name", "Library",
+					  "name", _("Library"),
 					  "container", container,
 					  "library", library,
 					  NULL));
@@ -611,19 +608,15 @@ album_node_selected_cb (RBNodeView *view,
 	songs_filter (source, genre, artist, node);
 }
 
-static const char *
-impl_get_description (RBSource *source)
-{
-	return _("Library");
-}
-
 static const char *
 impl_get_status (RBSource *asource)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	return rb_library_compute_status (source->priv->library,
-					  rb_library_get_all_songs (source->priv->library),
-					  source->priv->songs_filter);
+	g_free (source->priv->status);
+	source->priv->status = rb_library_compute_status (source->priv->library,
+							  rb_library_get_all_songs (source->priv->library),
+							  source->priv->songs_filter);
+	return source->priv->status;
 }
 
 static const char *
@@ -912,8 +905,15 @@ rb_library_source_add_location (RBLibrarySource *source, GtkWindow *win)
 	case GTK_RESPONSE_OK:
 	{
 		char *uri = gtk_editable_get_chars (GTK_EDITABLE (uri_widget), 0, -1);
-		rb_debug ("Got location \"%s\", adding...", uri);
-		rb_library_add_uri (source->priv->library, (char *) uri);
+		if (uri != NULL) {
+			GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
+			if (vfsuri != NULL) {
+				rb_library_add_uri (source->priv->library, uri);
+				gnome_vfs_uri_unref (vfsuri);
+			} else
+				rb_debug ("invalid uri: \"%s\"", uri);
+
+		}
 		break;
 	}
 	default:
